@@ -121,7 +121,8 @@ user:file_search_path(library, swi(packages)).
 user:file_search_path(foreign, AppDir) :-
     current_prolog_flag(windows, true),
     current_prolog_flag(executable, Exe),
-    file_directory_name(Exe, AppDir).
+    prolog_to_os_filename(PlExe, Exe),
+    file_directory_name(PlExe, AppDir).
 
 %!  add_package(+Package, +PkgSrcDir, +PkgBinDir) is det.
 %
@@ -131,7 +132,14 @@ user:file_search_path(foreign, AppDir) :-
 add_package(xpce, PkgBinDir) :-
     !,
     add_package_path(PkgBinDir),
-    use_module(swi('xpce/prolog/swipl-rc')).
+    (   absolute_file_name(swi('xpce/prolog/swipl-rc'),
+			   RCFile,
+			   [ access(read),
+			     file_errors(fail)
+			   ])
+    ->  use_module(RCFile)
+    ;   true
+    ).
 add_package(chr, PkgBinDir) :-
     assertz(user:file_search_path(chr, PkgBinDir)),
     assertz(user:file_search_path(chr, library(chr))),
@@ -149,20 +157,20 @@ add_package(_Pkg, PkgBinDir) :-
 
 %!  add_package_path(+PkgBinDir) is det.
 %
-%   Add the source  and  binary  directories   for  the  package  to the
-%   `library` and `foreign` search paths. Note that  we only need to add
-%   the binary directory if  it  contains   shared  objects,  but  it is
-%   probably cheaper to add it anyway.
+%   Add the binary directories for the   package to the `foreign` search
+%   path. Note that we only  need  to   add  the  binary directory if it
+%   contains shared objects, but  it  is   probably  cheaper  to  add it
+%   anyway. On Windows, all .dll  files  are   in  the  directory of the
+%   executable.
 
+add_package_path(_) :-
+    current_prolog_flag(windows, true),
+    !.
 add_package_path(PkgBinDir) :-
     assertz(user:file_search_path(foreign, PkgBinDir)).
 
-:- if(\+ current_prolog_flag(emscripten, true)).
-% disabled as we do not (yet) have packages and opendir() is broken
-% and this directory_files/2 raises an exception.
 :- forall(swipl_package(Pkg, PkgBinDir),
 	  add_package(Pkg, PkgBinDir)).
-:- endif.
 
 %!  set_version_info
 %
