@@ -3,7 +3,7 @@
     Author:        Jan Wielemaker
     E-mail:        J.Wielemaker@vu.nl
     WWW:           http://www.swi-prolog.org
-    Copyright (c)  2010-2021, VU University Amsterdam
+    Copyright (c)  2010-2023, VU University Amsterdam
                               CWI, Amsterdam
                               SWI-Prolog Solutions b.v.
     All rights reserved.
@@ -42,11 +42,7 @@
           ]).
 :- autoload(library(error), [domain_error/2, must_be/2, instantiation_error/1]).
 :- autoload(library(lists), [append/3]).
-:- autoload(library(uri), [uri_file_name/2]).
-:- if(exists_source(library(time))).
-:- autoload(library(time), [call_with_time_limit/2]).
-:- endif.
-
+:- autoload(library(utf8), [utf8_codes/3]).
 
 /** <module> Print decorated text to ANSI consoles
 
@@ -429,60 +425,95 @@ class_attrs(Attrs, Attrs).
 ansi_hyperlink(Stream, Location) :-
     hyperlink(Stream, url(Location)),
     !.
-ansi_hyperlink(Stream, File:Line:Column) :-
-    !,
-    (   url_file_name(URI, File)
-    ->  format(Stream, '\e]8;;~w#~d:~d\e\\~w:~d:~d\e]8;;\e\\',
-               [ URI, Line, Column, File, Line, Column ])
-    ;   format(Stream, '~w:~w:~w', [File, Line, Column])
-    ).
-ansi_hyperlink(Stream, File:Line) :-
-    !,
-    (   url_file_name(URI, File)
-    ->  format(Stream, '\e]8;;~w#~w\e\\~w:~d\e]8;;\e\\',
-               [ URI, Line, File, Line ])
-    ;   format(Stream, '~w:~w', [File, Line])
-    ).
-ansi_hyperlink(Stream, File) :-
-    (   url_file_name(URI, File)
-    ->  format(Stream, '\e]8;;~w\e\\~w\e]8;;\e\\',
-               [ URI, File ])
-    ;   format(Stream, '~w', [File])
-    ).
+ansi_hyperlink(Stream, Location) :-
+    location_label(Location, Label),
+    ansi_hyperlink(Stream, Location, Label).
 
-ansi_hyperlink(Stream, URL, Label) :-
-    hyperlink(Stream, url(URL, Label)),
-    !.
-ansi_hyperlink(Stream, URL, Label) :-
-    (   current_prolog_flag(hyperlink_term, true)
+location_label(File:Line:Column, Label) =>
+    format(string(Label), '~w:~w:~w', [File,Line,Column]).
+location_label(File:Line, Label) =>
+    format(string(Label), '~w:~w', [File,Line]).
+location_label(File, Label) =>
+    format(string(Label), '~w', [File]).
+
+ansi_hyperlink(Stream, Location, Label),
+    hyperlink(Stream, url(Location, Label)) =>
+    true.
+ansi_hyperlink(Stream, File:Line:Column, Label) =>
+    (   url_file_name(URI, File)
+    ->  format(Stream, '\e]8;;~w#~d:~d\e\\~w\e]8;;\e\\',
+               [ URI, Line, Column, Label ])
+    ;   format(Stream, '~w', [Label])
+    ).
+ansi_hyperlink(Stream, File:Line, Label) =>
+    !,
+    (   url_file_name(URI, File)
+    ->  format(Stream, '\e]8;;~w#~w\e\\~w\e]8;;\e\\',
+               [ URI, Line, Label ])
+    ;   format(Stream, '~w', [Label])
+    ).
+ansi_hyperlink(Stream, File, Label) =>
+    (   url_file_name(URI, File)
     ->  format(Stream, '\e]8;;~w\e\\~w\e]8;;\e\\',
-               [ URL, Label ])
+               [ URI, Label ])
     ;   format(Stream, '~w', [Label])
     ).
 
-%!  hyperlink(+Stream, +Spec) is semidet.
-%
-%   Multifile hook that may be used   to redefine ansi_hyperlink/2,3. If
-%   this predicate succeeds the system assumes the link has been written
-%   to Stream.
-%
-%   @arg  Spec  is  either  url(Location)    or   url(URL,  Label).  See
-%   ansi_hyperlink/2,3 for details.
+is_url(URL) :-
+    (   atom(URL)
+    ->  true
+    ;   string(URL)
+    ),
+    url_prefix(Prefix),
+    sub_string(URL, 0, _, _, Prefix).
 
-:- dynamic has_lib_uri/1 as volatile.
+url_prefix('http://').
+url_prefix('https://').
+url_prefix('file://').
+
+
+%!  url_file_name(-URL, +File) is semidet.
+%
+%   Same as uri_file_name/2 in mode (-,+), but   as a core library we do
+%   not wish to depend on the `clib` package and its foreign support.
 
+url_file_name(URL, File) :-
+    is_url(File), !,
+    URL = File.
 url_file_name(URL, File) :-
     current_prolog_flag(hyperlink_term, true),
-    (   has_lib_uri(true)
-    ->  uri_file_name(URL, File)
-    ;   exists_source(library(uri))
-    ->  use_module(library(uri), [uri_file_name/2]),
-        uri_file_name(URL, File),
-        asserta(has_lib_uri(true))
-    ;   asserta(has_lib_uri(false)),
-        fail
+    absolute_file_name(File, AbsFile),
+    ensure_leading_slash(AbsFile, AbsFile1),
+    url_encode_path(AbsFile1, Encoded),
+    format(string(URL), 'file://~s', [Encoded]).
+
+ensure_leading_slash(Path, SlashPath) :-
+    (   sub_atom(Path, 0, _, _, /)
+    ->  SlashPath = Path
+    ;   atom_concat(/, Path, SlashPath)
     ).
 
+url_encode_path(Name, Encoded) :-
+    atom_codes(Name, Codes),
+    phrase(utf8_codes(Codes), UTF8),
+    phrase(encode(UTF8), Encoded).
+
+encode([]) --> [].
+encode([H|T]) --> encode1(H), encode(T).
+
+encode1(C) -->
+    { reserved(C),
+      !,
+      format(codes([C1,C2]), '~`0t~16r~2|', [C])
+    },
+    "%", [C1,C2].
+encode1(C) -->
+    [C].
+
+reserved(C) :- C =< 0'\s.
+reserved(C) :- C >= 127.
+reserved(0'#).
+
 %!  keep_line_pos(+Stream, :Goal)
 %
 %   Run goal without changing the position   information on Stream. This
@@ -510,8 +541,6 @@ keep_line_pos(_, G) :-
 %   @arg RGB is a term rgb(Red,Green,Blue).  The color components are
 %   integers in the range 0..65535.
 
-
-:- if(current_predicate(call_with_time_limit/2)).
 ansi_get_color(Which0, RGB) :-
     stream_property(user_input, tty(true)),
     stream_property(user_output, tty(true)),
@@ -524,7 +553,7 @@ ansi_get_color(Which0, RGB) :-
     ),
     catch(keep_line_pos(user_output,
                         ansi_get_color_(Which, RGB)),
-          time_limit_exceeded,
+          error(timeout_error(_,_), _),
           no_xterm).
 
 supports_get_color :-
@@ -541,8 +570,11 @@ ansi_get_color_(Which, rgb(R,G,B)) :-
     hex4(GH),
     hex4(BH),
     phrase(("\e]", Id, ";rgb:", RH, "/", GH, "/", BH, "\a"), Pattern),
-    call_with_time_limit(0.05,
-                         with_tty_raw(exchange_pattern(Which, Pattern))),
+    stream_property(user_input, timeout(Old)),
+    setup_call_cleanup(
+        set_stream(user_input, timeout(0.05)),
+        with_tty_raw(exchange_pattern(Which, Pattern)),
+        set_stream(user_input, timeout(Old))),
     !,
     hex_val(RH, R),
     hex_val(GH, G),
@@ -595,13 +627,6 @@ echo([H|T]) :-
     put_code(user_output, H),
     echo(T).
 
-:- else.
-ansi_get_color(_Which0, _RGB) :-
-    fail.
-:- endif.
-
-
-
 :- multifile prolog:message//1.
 
 prolog:message(ansi(no_xterm_get_colour)) -->
