@@ -230,7 +230,7 @@ pl_break1(atom_t goal)
   IOSTREAM *outSave      = Scurout;
   intptr_t skipSave      = debugstatus.skiplevel;
   int  suspSave          = debugstatus.suspendTrace;
-  int  traceSave;
+  bool traceSave;
   debug_type debugSave;
   tbl_status tblstat;
 
@@ -566,7 +566,8 @@ abortProlog(void)
   term_t ex;
   bool rc = false;
 
-  pl_notrace();
+  tracemode(false, NULL);
+  debugmode(DBG_OFF, NULL);
   Sreset();				/* Discard pending IO */
 
   LD->exception.processing = true;	/* allow using spare stack */
