@@ -3,7 +3,7 @@
     Author:        Jan Wielemaker
     E-mail:        J.Wielemaker@vu.nl
     WWW:           http://www.swi-prolog.org
-    Copyright (c)  1999-2024, University of Amsterdam
+    Copyright (c)  1999-2025, University of Amsterdam
                               VU University Amsterdam
 			      CWI, Amsterdam
 			      SWI-Prolog Solutions b.v.
@@ -514,6 +514,12 @@ static inline	PL_local_data_t *acquire_ldata(DECL_LD PL_thread_info_t *info);
 #define HAVE_PTHREAD_EXIT 1
 #endif
 
+#if USE_LD_MACROS
+#define localiseDefinition(def)		LDFUNC(localiseDefinition, def)
+#define unify_thread_id(id, info)	LDFUNC(unify_thread_id, id, info)
+#endif /*USE_LD_MACROS*/
+#define LDFUNC_DECLARATIONS
+
 int		exitPrologThreads(void);
 bool		aliasThread(int tid, atom_t type, atom_t name);
 foreign_t	pl_thread_create(term_t goal, term_t id,
@@ -521,7 +527,7 @@ foreign_t	pl_thread_create(term_t goal, term_t id,
 foreign_t	pl_thread_at_exit(term_t goal);
 int		PL_thread_self(void);
 #ifdef O_ENGINES
-int		unify_thread_id(term_t id, PL_thread_info_t *info);
+bool		unify_thread_id(term_t id, PL_thread_info_t *info);
 #endif
 int		enableThreads(int enable);
 
@@ -543,6 +549,7 @@ intptr_t	system_thread_id(PL_thread_info_t *info);
 void		get_current_timespec(struct timespec *time);
 void		carry_timespec_nanos(struct timespec *time);
 void		free_predicate_references(PL_local_data_t *ld);
+#undef LDFUNC_DECLARATIONS
 
 
 		 /*******************************
@@ -613,28 +620,27 @@ typedef struct
 #define require_c_stack(needed)		LDFUNC(require_c_stack, needed)
 #define clear_low_c_stack(_)		LDFUNC(clear_low_c_stack, _)
 #endif /*USE_LD_MACROS*/
-
 #define LDFUNC_DECLARATIONS
 
 int		get_prop_def(term_t t, atom_t expected,
 			     const tprop *list, const tprop **def);
 void		initPrologThreads(void);
-int		pl_atom_table_in_use(AtomTable atom_table);
-int		pl_atom_bucket_in_use(Atom *atom_bucket);
+bool		pl_atom_table_in_use(AtomTable atom_table);
+bool		pl_atom_bucket_in_use(Atom *atom_bucket);
 Atom**		pl_atom_buckets_in_use(void);
 Definition*	predicates_in_use(void);
-int		pl_functor_table_in_use(FunctorTable functor_table);
-int		pl_kvs_in_use(KVS kvs);
+bool		pl_functor_table_in_use(FunctorTable functor_table);
+bool		pl_kvs_in_use(KVS kvs);
 definition_ref* pushPredicateAccessObj(Definition def);
 void		popPredicateAccess(Definition def);
 size_t		popNPredicateAccess(size_t n);
 void		markAccessedPredicates(PL_local_data_t *ld);
-int		cgc_thread_stats(cgc_stats *stats);
-int		signalGCThread(int sig);
-int		isSignalledGCThread(int sig);
+bool		cgc_thread_stats(cgc_stats *stats);
+bool		signalGCThread(int sig);
+bool		isSignalledGCThread(int sig);
 double		ThreadCPUTime(int which);
 void		updatePendingThreadSignals(void);
-int		require_c_stack(size_t needed);
+bool		require_c_stack(size_t needed);
 void		clear_low_c_stack(void);
 
 #undef LDFUNC_DECLARATIONS
